/**
********************************************************************************
 * @file   main.c
 * @brief  the application functions of KWUP demo for the TOSHIBA
             'TMPM362' Device Series 
 * @version V0.101
 * @date    2014/05/21
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
/* Includes ------------------------------------------------------------------*/
#include "tmpm362_kwup.h"
#include "led.h"

void INTKWUP_IRQHandler(void);
int main(void)
{
    KWUP_SettingTypeDef keySetting;
    KWUP_PortStatus portStatus;
    SystemInit();

    /* set PJ4,PJ5 to Key0,1 */
    TSB_PJ_FR2_PJ4F2 = 1U;
    TSB_PJ_PUP_PJ4UP = 1U;
    TSB_PJ_IE_PJ4IE = 1U;

    TSB_PJ_FR2_PJ5F2 = 1U;
    TSB_PJ_PUP_PJ5UP = 1U;
    TSB_PJ_IE_PJ5IE = 1U;

    /* Set PM4 to Output Port */
    LED_Configuration();
    LED_On(LED1);

    /* enable KWUP interrupt */
    NVIC_ClearPendingIRQ(INTKWUP_IRQn);
    NVIC_EnableIRQ(INTKWUP_IRQn);

    keySetting.KeyN = KWUP_INPUT_0;
    keySetting.PullUpCtrl = KWUP_PUP_CTRL_BY_DYNAMIC;
    keySetting.ActiveState = KWUP_ACTIVE_BY_RISING_EDGE;
    keySetting.INTNewState = ENABLE;
    KWUP_SetConfig(&keySetting);

    KWUP_SetPullUpConfig(KWUP_CYCLES_4_FS, KWUP_CYCLES_256_FS);

    KWUP_ClearINTReq();

    /* enable low power mode release interrupt to KWUP */
    TSB_CG->IMCGF = 0x1000U;
    TSB_CG->IMCGF += 0x0100U;

    do {
        portStatus = KWUP_GetPortStatus();

        if (portStatus.Bit.Key1 == 0U) {        /* press key(PJ5) */
            LED_Off(LED1);
            /* enter low power mode */
            __WFI();

            /* press KEY(PJ4) to release low power mode */

            LED_On(LED1);
        }
    }
    while (1);

}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif


void INTKWUP_IRQHandler(void)
{
    #define CG_INT_SRC_KWUP 21U
    KWUP_ClearINTReq();
    TSB_CG->ICRCG = CG_INT_SRC_KWUP;
}
