/** 
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of UART0 to UART2 demo for the TOSHIBA
 *          'TMPM362' Device Series 
 * @version V0.103
 * @date    2014/05/21
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "tmpm362_uart.h"
#include <stdio.h>

#define BufSize (sizeof(TxBuffer) / sizeof(*(TxBuffer)))
#define ON 0
#define OFF 1
#define LED4 TSB_PM_DATA_PM4
#define LED5 TSB_PM_DATA_PM5
#define LED6 TSB_PM_DATA_PM6
#define LED7 TSB_PM_DATA_PM7

typedef enum { KEY7 = 1, KEY6, KEY5, KEY4 } KEYID;
uint8_t TxBuffer[] = "TMPM362";
uint8_t RxBuffer[BufSize] = { 0U };

const uint8_t NumToBeTx = BufSize - 1U;
uint8_t TxCounter = 0U;
uint8_t RxCounter = 0U;
UART_InitTypeDef myUART;
uint8_t KeyNum = 0U;

void SIO_Configuration(TSB_SC_TypeDef * SCx);
void Key_Configuration(void);
void Key_WaitRelease(KEYID key);
void Led_Init(void);
void Led_AllOff(void);
void ResetIdx(void);

int main(void)
{
    Key_Configuration();
    Led_Init();
    SIO_Configuration(UART0);
    SIO_Configuration(UART2);

    myUART.BaudRate = (uint32_t) 115200;
    myUART.DataBits = UART_DATA_BITS_8;
    myUART.StopBits = UART_STOP_BITS_1;
    myUART.Parity = UART_NO_PARITY;
    myUART.Mode = UART_ENABLE_RX | UART_ENABLE_TX;
    myUART.FlowCtrl = UART_NONE_FLOW_CTRL;

    UART_Enable(UART0);
    UART_Init(UART0, &myUART);

    UART_Enable(UART2);
    UART_Init(UART2, &myUART);
    
    NVIC_EnableIRQ(INTTX0_IRQn);
    NVIC_EnableIRQ(INTRX2_IRQn);
    
    do {
        Key_WaitRelease(KEY4);
        
        UART_SetTxData(UART0, (uint32_t) (TxBuffer[TxCounter++]));
        while (RxCounter < NumToBeTx) {
            /* Do nothing */
        }
#ifdef DEBUG        
        printf("%s\n", RxBuffer);
#endif        
        Key_WaitRelease(KEY4);
        ResetIdx();
    } while (1);

}

void ResetIdx(void)
{
    uint8_t i = 0U;

    for (i = 0U; i <= RxCounter; i++) {
        RxBuffer[i] = 0U;
    }
    TxCounter = 0U;
    RxCounter = 0U;
}

void SIO_Configuration(TSB_SC_TypeDef * SCx)
{
    if (SCx == TSB_SC0) {
        TSB_PE_CR_PE4C = 1;
        TSB_PE_FR2_PE4F2 = 1;
        TSB_PE_FR2_PE5F2 = 1;
        TSB_PE_IE_PE5IE = 1;
    } else if (SCx == TSB_SC1) {
        TSB_PL_CR_PL4C = 1;
        TSB_PL_FR1_PL4F1 = 1;
        TSB_PL_FR1_PL5F1 = 1;
        TSB_PL_IE_PL5IE = 1;
    } else if (SCx == TSB_SC2) {
        TSB_PM_CR_PM1C = 1;
        TSB_PM_FR1_PM1F1 = 1;
        TSB_PM_FR1_PM2F1 = 1;
        TSB_PM_IE_PM2IE = 1;
    } else if (SCx == TSB_SC3) {
        TSB_PM_CR_PM5C = 1;
        TSB_PM_FR1_PM5F1 = 1;
        TSB_PM_FR1_PM6F1 = 1;
        TSB_PM_IE_PM6IE = 1;
    } else if (SCx == TSB_SC4) {
        TSB_PN_CR_PN0C = 1;
        TSB_PN_FR1_PN0F1 = 1;
        TSB_PN_FR1_PN1F1 = 1;
        TSB_PN_IE_PN1IE = 1;
    } else if (SCx == TSB_SC5) {
        TSB_PN_CR_PN4C = 1;
        TSB_PN_FR1_PN4F1 = 1;
        TSB_PN_FR1_PN5F1 = 1;
        TSB_PN_IE_PN5IE = 1;
    } else if (SCx == TSB_SC6) {
        TSB_PO_CR_PO0C = 1;
        TSB_PO_FR1_PO0F1 = 1;
        TSB_PO_FR1_PO1F1 = 1;
        TSB_PO_IE_PO1IE = 1;
    } else if (SCx == TSB_SC7) {
        TSB_PO_CR_PO4C = 1;
        TSB_PO_FR1_PO4F1 = 1;
        TSB_PO_FR1_PO5F1 = 1;
        TSB_PO_IE_PO5IE = 1;
    } else if (SCx == TSB_SC8) {
        TSB_PC_CR_PC0C = 1;
        TSB_PC_FR2_PC0F2 = 1;
        TSB_PC_FR2_PC1F2 = 1;
        TSB_PC_IE_PC1IE = 1;
    } else if (SCx == TSB_SC9) {
        TSB_PC_CR_PC4C = 1;
        TSB_PC_FR2_PC4F2 = 1;
        TSB_PC_FR2_PC5F2 = 1;
        TSB_PC_IE_PC5IE = 1;
    } else if (SCx == TSB_SC10) {
        TSB_PD_CR_PD0C = 1;
        TSB_PD_FR2_PD0F2 = 1;
        TSB_PD_FR2_PD1F2 = 1;
        TSB_PD_IE_PD1IE = 1;
    } else if (SCx == TSB_SC11) {
        TSB_PD_CR_PD4C = 1;
        TSB_PD_FR2_PD4F2 = 1;
        TSB_PD_FR2_PD5F2 = 1;
        TSB_PD_IE_PD5IE = 1;
    }
}

void Led_AllOff(void)
{
    LED4 = OFF;
    LED5 = OFF;
    LED6 = OFF;
    LED7 = OFF;
}

void Led_Init(void)
{
    TSB_PM_DATA_PM4 = 1;
    TSB_PM_DATA_PM5 = 1;
    TSB_PM_DATA_PM6 = 1;
    TSB_PM_DATA_PM7 = 1;
    TSB_PM_CR_PM4C = 1;
    TSB_PM_CR_PM5C = 1;
    TSB_PM_CR_PM6C = 1;
    TSB_PM_CR_PM7C = 1;
}

void Key_Configuration(void)
{
    TSB_PJ->IE |= (uint32_t) 0x00F0;
    TSB_PJ->PUP |= 0x00F0;
}

void Key_WaitRelease(KEYID key)
{
    switch (key) {
    case KEY7:
        while (TSB_PJ_DATA_PJ7);
        while (!TSB_PJ_DATA_PJ7);
        LED7 = ON;
        break;
    case KEY6:
        while (TSB_PJ_DATA_PJ6);
        while (!TSB_PJ_DATA_PJ6);
        LED6 = ON;
        break;
    case KEY5:
        while (TSB_PJ_DATA_PJ5);
        while (!TSB_PJ_DATA_PJ5);
        LED5 = ON;
        break;
    case KEY4:
        while (TSB_PJ_DATA_PJ4);
        while (!TSB_PJ_DATA_PJ4);
        LED4 = ON;
        break;
    default:
        break;
    }
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    printf("Err!");
    while (1) {
        __NOP();
    }
}
#endif
