/**
 *******************************************************************************
 * @file    tmpm362_uart_int.c
 * @brief   all interrupt request functions definition of UART (Serial Channel)
 *          for the TOSHIBA 'TMPM362' Device Series 
 * @version V0.103
 * @date    2014/05/21
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm362_uart_int.h"

extern const uint8_t NumToBeTx;
extern uint8_t TxBuffer[];
extern uint8_t RxBuffer[];
extern uint8_t TxCounter;
extern uint8_t RxCounter;

/**
  * @brief  The transmission interrupt request function of UART channel 0.
  * @param  None
  * @retval None
  */
void INTTX0_IRQHandler(void)
{
    volatile UART_Err err;

    if (TxCounter < NumToBeTx) {
        UART_SetTxData(UART0, TxBuffer[TxCounter++]);
    } else {
        err = UART_GetErrState(UART0);
    }
}

/**
  * @brief  The reception interrupt request function of UART channel 2.
  * @param  None
  * @retval None
  */
void INTRX2_IRQHandler(void)
{
    volatile UART_Err err;

    err = UART_GetErrState(UART2);
    if (UART_NO_ERR == err) {
        RxBuffer[RxCounter++] = (uint8_t) UART_GetRxData(UART2);
    }
}
